/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This Source Code Form is "Incompatible With Secondary Licenses", as
 * defined by the Mozilla Public License, v. 2.0.
 */

/* These are specified using the class instead of the id so that they
   don't override the YUI CSS. */
.enter_bug_form table {
    border-spacing: 0;
    border-width: 0;
}
.enter_bug_form td, .enter_bug_form th { padding: .25em; }
.enter_bug_form th { text-align: right; }

/* This makes the "component" column as small as possible (since it
 * contains only fixed-width content) and the Reporter column
 * as large as possible, which makes the form not jump around
 * when the Component Description changes size. This works
 * pretty well on all browsers except IE 8.
 */
#Create #field_container_component { width: 1px; }
#Create #field_container_reporter  { width: 100%; }

#Create .comment {
    vertical-align: top;
    overflow: auto;
    color: green;
}
#Create #comp_desc_container td { padding: 0; }
#Create #comp_desc { height: 11ex; }
#Create #os_guess_note {
    padding-top: 0;
}
#Create #os_guess_note div {
    max-width: 35em;
}

/* Text inputs need to be a little shorter on enter_bug
 * than the 100% that they are on show_bug.
 */
#Create .field_value .text_input { max-width: 50em; }

/* The Possible Duplicates table on enter_bug. */
#possible_duplicates th {
    text-align: center; 
    background: none;
    border-collapse: collapse;
}
/* Make the Add Me to CC button never wrap. */
#possible_duplicates .yui-dt-col-update_token { white-space: nowrap; }

form#Create #possible_duplicates td { vertical-align: middle; }
